@@ -142,9 +142,10 @@ static void init_control_bitmap (reiserfs_filsys_t * fs)
     unsigned int i;
     unsigned long block;
     unsigned long reserved;
+    unsigned int blocks = get_sb_block_count(fs->fs_ondisk_sb);
     
 
-    control_bitmap = reiserfs_create_bitmap(get_sb_block_count(fs->fs_ondisk_sb));
+    control_bitmap = reiserfs_create_bitmap(blocks);
     if (!control_bitmap)
 	die ("init_control_bitmap: Failed to allocate a control bitmap.");
 
@@ -161,7 +162,7 @@ static void init_control_bitmap (reiserfs_filsys_t * fs)
 
     /* bitmaps */
     block = fs->fs_super_bh->b_blocknr + 1;
-    for (i = 0; i < get_sb_bmap_nr (fs->fs_ondisk_sb); i ++) {
+    for (i = 0; i < reiserfs_fs_bmap_nr(fs); i ++) {
         we_met_it (block);
 
 	if (spread_bitmaps (fs))
@@ -394,11 +395,11 @@ static int bad_direct_item (reiserfs_filsys_t * fs,
 
 inline void handle_one_pointer (reiserfs_filsys_t * fs, 
 				struct buffer_head * bh,
-				__u32 * ptr) 
+				__u32 * item, int offset)
 {
     if (fsck_mode (fs) == FSCK_FIX_FIXABLE) {
 	fsck_log (" - zeroed");
-	*ptr = 0;
+	d32_put (item, offset, 0);
 	mark_buffer_dirty (bh);
     } else {
 	one_more_corruption (fs, FIXABLE);
@@ -423,7 +424,7 @@ static int bad_badblocks_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 	    return 0;
     
     for (i = 0; i < I_UNFM_NUM (ih); i ++) {
-	if (!le32_to_cpu (ind [i])) {
+	if (!d32_get (ind, i)) {
 /*	    fsck_log ("%s: block %lu: badblocks item (%H) has zero pointer.",
 		      __FUNCTION__, bh->b_blocknr, ih);
 	    
@@ -437,14 +438,14 @@ static int bad_badblocks_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 	}
 
 	/* check list of badblocks pointers */
-	if (le32_to_cpu (ind [i]) >= get_sb_block_count (fs->fs_ondisk_sb)) {
+	if (d32_get (ind, i) >= get_sb_block_count (fs->fs_ondisk_sb)) {
 	    fsck_log ("%s: badblock pointer (block %lu) points out of disk spase (%lu)",
-			__FUNCTION__, bh->b_blocknr, le32_to_cpu (ind [i]));
-	    handle_one_pointer (fs, bh, &ind[i]);
+			__FUNCTION__, bh->b_blocknr, d32_get (ind, i));
+	    handle_one_pointer (fs, bh, ind, i);
 	    fsck_log ("\n");
 	}
 
-	if (did_we_meet_it (le32_to_cpu (ind [i]))) {
+	if (did_we_meet_it (d32_get (ind, i))) {
 	    /* it can be
 	       1. not_data_block
 	       		delete pointer
@@ -452,23 +453,23 @@ static int bad_badblocks_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
 	          advice to run fix-fixable if there is no fatal errors
 	          with list of badblocks, say that it could fix it. */
 	
-	    if (not_data_block (fs, le32_to_cpu (ind [i]))) {
+	    if (not_data_block (fs, d32_get (ind, i))) {
 		fsck_log ("%s: badblock pointer (block %lu) points on fs metadata (%lu)",
-			  __FUNCTION__, bh->b_blocknr, le32_to_cpu (ind [i]));
-		handle_one_pointer (fs, bh, &ind[i]);
+			  __FUNCTION__, bh->b_blocknr, d32_get (ind, i));
+		handle_one_pointer (fs, bh, ind, i);
 		fsck_log ("\n");
 	    } else {
 		one_more_corruption(fs, FIXABLE);
 	        fsck_log ("%s: badblock pointer (block %lu) points to a block (%lu) "
 			  "which is in the tree already. Use badblock option (-B) to"
 			  " fix the problem\n", __FUNCTION__, bh->b_blocknr, 
-			  le32_to_cpu(ind[i]));
+			  d32_get (ind, i));
 	    }
 
 	    continue;
 	}
 	
-	we_met_it(le32_to_cpu(ind[i]));
+	we_met_it(d32_get(ind, i));
     }
 
     return 0;
@@ -490,33 +491,31 @@ static int bad_indirect_item (reiserfs_filsys_t * fs, struct buffer_head * bh,
     }
 
     for (i = 0; i < I_UNFM_NUM (ih); i ++) {
-//	__u32 unfm_ptr;
 
 	fsck_check_stat (fs)->unfm_pointers ++;
-//	unfm_ptr = le32_to_cpu (ind [i]);
-	if (!le32_to_cpu (ind [i])) {
+	if (!d32_get (ind, i)) {
 	    fsck_check_stat (fs)->zero_unfm_pointers ++;
 	    continue;
 	}
 
 	/* check unformatted node pointer and mark it used in the
            control bitmap */
-	if (bad_block_number (fs, le32_to_cpu (ind [i]))) {
+	if (bad_block_number (fs, d32_get (ind, i))) {
 	    fsck_log ("%s: block %lu: The item %k has the bad pointer (%d) to "
 		      "the block (%lu)", __FUNCTION__, bh->b_blocknr, 
-		      &ih->ih_key, i, le32_to_cpu (ind [i]));
+		      &ih->ih_key, i, d32_get (ind, i));
 	    
-	    handle_one_pointer (fs, bh, &ind[i]);
+	    handle_one_pointer (fs, bh, ind, i);
 	    fsck_log ("\n");
 	    continue;
 	}
 
-        if (got_already (fs, le32_to_cpu (ind [i]))) {
+        if (got_already (fs, d32_get (ind, i))) {
 	    fsck_log ("%s: block %lu: The item (%H) has the bad pointer (%d) "
 		      "to the block (%lu), which is in tree already", 
-		      __FUNCTION__, bh->b_blocknr, ih, i, le32_to_cpu(ind[i]));
+		      __FUNCTION__, bh->b_blocknr, ih, i, d32_get (ind, i));
 	    
-	    handle_one_pointer (fs, bh, &ind [i]);
+	    handle_one_pointer (fs, bh, ind, i);
 	    fsck_log ("\n");
             continue;
 	}
@@ -543,8 +542,8 @@ static int bad_directory_item (reiserfs_filsys_t * fs,
 			       struct buffer_head * bh, 
 			       struct item_head * ih)
 {
-    char * name;
-    int namelen;
+    char *name, *prev_name;
+    __u32 off, prev_off;
     unsigned int count, i;
     struct reiserfs_de_head * deh = B_I_DEH (bh, ih);
     int min_entry_size = 1;/* We have no way to understand whether the
@@ -552,6 +551,7 @@ static int bad_directory_item (reiserfs_filsys_t * fs,
                               converted to it. So, we assume that minimal name
                               length is 1 */
     __u16 state;
+    int namelen;
 
     count = get_ih_entry_count (ih);
 
@@ -582,17 +582,34 @@ static int bad_directory_item (reiserfs_filsys_t * fs,
     }
     
     /* check name hashing */
+    prev_name = B_I_PITEM(bh, ih) + get_ih_item_len(ih);
+    prev_off = 0;
+
     for (i = 0; i < count; i ++, deh ++) {
 	namelen = name_in_entry_length (ih, deh, i);
 	name = name_in_entry (deh, i);
-	if (!is_properly_hashed (fs, name, namelen, get_deh_offset (deh))) {
+	off = get_deh_offset (deh);
+	
+	if (namelen > (int)REISERFS_MAX_NAME_LEN(fs->fs_blocksize) || 
+	    name >= prev_name || off <= prev_off) 
+	{
+	    fsck_log ("%s: block %lu: The directory item %k has a broken entry "
+		      "(%d)\n", __FUNCTION__, bh->b_blocknr, &ih->ih_key, i);
+	    one_more_corruption (fs, FATAL);
+	    return 1;
+	}
+
+	if (!is_properly_hashed (fs, name, namelen, off)) {
 	    fsck_log ("%s: block %lu: The directory item %k has a not properly "
 		      "hashed entry (%d)\n", __FUNCTION__, bh->b_blocknr, 
 		      &ih->ih_key, i);
 	    
-	    one_more_corruption (fs, FATAL);	
+	    one_more_corruption (fs, FATAL);
 	    return 1;
 	}
+
+	prev_name = name;
+	prev_off = off;
     }
 
     deh = B_I_DEH (bh, ih);
@@ -1108,7 +1125,7 @@ void check_fs_tree (reiserfs_filsys_t * fs)
 {
     before_check_fs_tree (fs);
     
-    fsck_progress ("Checking internal tree..");
+    fsck_progress ("Checking internal tree..  ");
     pass_through_tree (fs, bad_node, bad_path, fsck_mode(fs) == FSCK_AUTO ? 2 : -1);
     /* internal tree is correct (including all objects have correct
        sequences of items) */
